// -*- C++ -*-
//
// userana.cc
// example code
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Sep 25 22:32:13 JST 2001
//
// $Id: userana.cc,v 1.1.1.1 2002/08/13 12:48:13 kensh Exp $
//
// Revision History
//
// $Log: userana.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:13  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <math.h>
#include <stdlib.h>
#include "CLHEP/Vector/ThreeVector.h"
#include "CLHEP/Vector/LorentzVector.h"
#include "TupleManager/TupleManager.h"
#include "ntuple.h"


//////
#define DEBUG 0

#define HLIM  10000000
int  pawc_[ HLIM ];

//
const static double  mass_pi0 = 0.1349764;


//////
int
pi0Vertex( const Hep3Vector& vgam1, double E1,
           const Hep3Vector& vgam2, double E2,
           double z[2] )
{
  // calculate pi0 vertex(Z) on the beam axis.
  // Mpi0^2 = 2 E1 E2 ( 1 - cos(th) ) 
  double  R1  = vgam1.perp2();
  double  R2  = vgam2.perp2();
  double  I   = vgam1.x()*vgam2.x() + vgam1.y()*vgam2.y();
  double  CT  = 1 - mass_pi0*mass_pi0/(2*E1*E2);
  double  CT2 = CT*CT;
  double  A   = 1 - CT2;
  double  B   = 2*I - CT2*(R1+R2);
  double  C   = I*I - CT2*R1*R2;
  double  D   = B*B - 4*A*C;
  int  ct = 0;
  if ( D < 0 ){
    std::cout << "pi0Vertex : cannot solve : D=" << D << std::endl;
  }
  else{
    double  d2p = (-B + sqrt(D))/2/A;
    double  d2m = (-B - sqrt(D))/2/A;
    if ( d2p > 0 )
      z[ct++] = vgam1.z() - sqrt( d2p );
    if ( d2m > 0 )
      z[ct++] = vgam1.z() - sqrt( d2m );
    if ( ct >= 2 && z[0] > z[1] ){
      std::swap( z[0],z[1] );
      std::cout << "pi0Vertex : z+ and z- swapped." << std::endl;
    }
  }
  return( ct );
}



//////
int
main( int argc, char* argv[] )
{
  if ( argc < 4 ){
    std::cout << "usage : userana outfile Nevent infiles..." << std::endl;
    return( 0 );
  }
  
  // init ZEBRA
  TupleManager::init( HLIM );
  
  // output
  TupleManager  ofile;
  ofile.wopen( argv[1],"OUT",21 );
  TMNtuple*  ntPi0 = ofile.ntuple( 2001,"PI0","NRESULT",&pi0data,s_pi0data );
  
  // input
  int  ct = 0;
  for( int f=3;f<argc;f++ ){
    TupleManager  ifile;
    ifile.ropen( argv[f],"IN",22 );
    TMNtuple*  ntResult = ifile.ntuple( 1001,"NRESULT",&nresult );
    int  nevent = ntResult->totalEntries();
    std::cout << "Total Entries in Gen = " << nevent << std::endl;
    
    // event loop
    int  ct = 1;
    int  nlimit = atoi( argv[2] );
    for( int i=1; i<=nevent && (nlimit<=0 || i<=nlimit); i++ ){
      ntResult->read( i );
      NRESULT&  gp = nresult;
      //std::cout << ++ct
      //          << " : Event" << i << " PN" << gp.PN[0] 
      //          << " MODE" << gp.STATUS[0] << " NGEN" << gp.NGEN
      //          << std::endl;
      if ( gp.PN[0]      == 10 && // KL
           gp.GSTATUS[0] == 10 && // KL->pi0 nu nubar
           gp.NGEN       == 4  ){ // KL,pi0,gm,gm
        // pi0
        Hep3Vector  v_pi0( gp.XX[1], gp.YY[1], gp.ZZ[1] );
        Hep3Vector  p3_pi0( gp.PX[1], gp.PY[1], gp.PZ[1] ); //unit vector
        p3_pi0.setMag( gp.PP[1] ); // magnitude
        double  e_pi0 = sqrt(p3_pi0.mag2() + mass_pi0*mass_pi0);
        HepLorentzVector  p_pi0( p3_pi0,e_pi0 );
#if DEBUG        
        std::cout << ct
                  << " : Event" << i << " PI0 : v" << v_pi0
                  << " p" << p_pi0 << std::endl;
#endif

        // 2 gammas
        Hep3Vector  v_g1(  gp.XX[2], gp.YY[2], gp.ZZ[2] );
        Hep3Vector  p3_g1( gp.PX[2], gp.PY[2], gp.PZ[2] ); //unit vector
        p3_g1.setMag( gp.PP[2] ); // magnitude
        HepLorentzVector  p_g1( p3_g1,p3_g1.mag() );
#if DEBUG        
        std::cout << ct
                  << " : Event" << i << " GM1 : v" << v_g1 
                  << " p" << p_g1 << std::endl;
#endif
        
        Hep3Vector  v_g2(  gp.XX[3], gp.YY[3], gp.ZZ[3] );
        Hep3Vector  p3_g2( gp.PX[3], gp.PY[3], gp.PZ[3] ); //unit vector
        p3_g2.setMag( gp.PP[3] ); // magnitude
        HepLorentzVector  p_g2( p3_g2,p3_g2.mag() );
#if DEBUG        
        std::cout << ct
                  << " : Event" << i << " GM2 : v" << v_g2 
                  << " p" << p_g2 << std::endl;
#endif

        // vertex
        pi0data = pi0data_clear;
        if ( v_g1.z() < 610 ){
          Hep3Vector h1( v_g1.x() + p3_g1.x()/p3_g1.z()*(610-v_g1.z()),
                         v_g1.y() + p3_g1.y()/p3_g1.z()*(610-v_g1.z()),
                         610 );
          Hep3Vector h2( v_g2.x() + p3_g2.x()/p3_g2.z()*(610-v_g2.z()),
                         v_g2.y() + p3_g2.y()/p3_g2.z()*(610-v_g2.z()),
                         610 );
          double  eg1 = p_g1.e();
          double  eg2 = p_g2.e();
          double  z0[2] = {0,0};
          int  nz = pi0Vertex( h1,eg1, h2,eg2, z0 );
          HepLorentzVector  p_recpi0_m,p_recpi0_p;
          if ( nz <= 0 )
            std::cout << "Event" << i << " : pi0 recon failed" << std::endl;
          else{
            Hep3Vector  p3_recg1_m( h1.x(),h1.y(),h1.z()-z0[0] );
            p3_recg1_m.setMag( eg1 );
            Hep3Vector  p3_recg2_m( h2.x(),h2.y(),h2.z()-z0[0] );
            p3_recg2_m.setMag( eg2 );
            Hep3Vector  p3_recpi0_m = p3_recg1_m + p3_recg2_m;
            double  e_recpi0_m = sqrt(p3_recpi0_m.mag2()+mass_pi0*mass_pi0);
            p_recpi0_m = HepLorentzVector( p3_recpi0_m, e_recpi0_m );
#if DEBUG        
            std::cout << "Event" << i << " : recpi0 : z=" << z0[0]
                      << " p=" << p_recpi0_m
                      << std::endl;
#endif
            if ( nz == 2 ){
              Hep3Vector  p3_recg1_p( h1.x(),h1.y(),h1.z()-z0[1] );
              p3_recg1_p.setMag( eg1 );
              Hep3Vector  p3_recg2_p( h2.x(),h2.y(),h2.z()-z0[1] );
              p3_recg2_p.setMag( eg2 );
              Hep3Vector  p3_recpi0_p = p3_recg1_p + p3_recg2_p;
              double  e_recpi0_p = sqrt(p3_recpi0_p.mag2()+mass_pi0*mass_pi0);
              p_recpi0_p = HepLorentzVector( p3_recpi0_p, e_recpi0_p );
#if DEBUG        
              std::cout << "Event" << i << " : recpi0 : z=" << z0[1]
                        << " p=" << p_recpi0_p
                        << std::endl;
#endif
            }
          }
          
          // fill ntuple
          pi0data.EVTNO       = i;
          pi0data.STATUS      = nz;
          pi0data.NPI0        = 1;
          pi0data.ZPI0[0][0]  = z0[0];
          pi0data.ZPI0[0][1]  = z0[1];
          pi0data.PXPI0[0][0] = p_recpi0_m.vect().x();
          pi0data.PYPI0[0][0] = p_recpi0_m.vect().y();
          pi0data.PZPI0[0][0] = p_recpi0_m.vect().z();
          pi0data.PPPI0[0][0] = p_recpi0_m.e();
          pi0data.PTPI0[0][0] = p_recpi0_m.vect().perp();
          pi0data.PXPI0[0][1] = p_recpi0_p.vect().x();
          pi0data.PYPI0[0][1] = p_recpi0_p.vect().y();
          pi0data.PZPI0[0][1] = p_recpi0_p.vect().z();
          pi0data.PPPI0[0][1] = p_recpi0_p.e();
          pi0data.PTPI0[0][1] = p_recpi0_p.vect().perp();
          pi0data.GXPI0       = v_pi0.x();
          pi0data.GYPI0       = v_pi0.y();
          pi0data.GZPI0       = v_pi0.z();
          pi0data.GPXPI0      = p3_pi0.x();
          pi0data.GPYPI0      = p3_pi0.y();
          pi0data.GPZPI0      = p3_pi0.z();
          pi0data.GPPPI0      = p_pi0.e();
          pi0data.GPTPI0      = p3_pi0.perp();
          ntPi0->dump();

#if DEBUG        
          std::cout << ct
                    << " : Event" << i << " RECON_PI0 :"
                    << " z=" << z0[0] << "," << z0[1]
                    << " (z1=" << h1.z() << ")"
                    << " p" << p_pi0 << std::endl;
#endif
          // 
          ct++;
        }
      }
    }
    // close
    ifile.close();
  }
  
  // close
  ofile.close();

  //
  return( 0 );
}
