// -*- C++ -*-
//
// user_ana.cc
//
// Author:  Ken Sakashita
// Created: Tue Aug 27 17:55:55 JST 2002
//
// $Id: user_ana.cc,v 1.1 2002/08/30 02:12:14 kensh Exp $
//
// Revision History
//
// $Log: user_ana.cc,v $
// Revision 1.1  2002/08/30 02:12:14  kensh
//
//
// New release. event dump tool.
//
//

#include <iostream>
#include <iomanip>
#include <list>
#include "midas.h"
#include "MidasManager/MidasBank.h"
#include "MidasManager/MidasFileIO.h"

typedef struct
{
  int   Eventid;
  bool  Dumpevent;
  int   Nevent;
  int   Skipevent;

} DUMP_OPTIONS;
extern DUMP_OPTIONS options;

//
static char* tname[] = {
  "?","BYTE","SBYTE","CHAR","WORD",
  "SHORT","DWORD","INT","BOOL","FLOAT",
  "DOUBLE","BITFIELD","STRING","ARRAY","STRUCT",
  "KEY","LINK","LAST",
};

static int  tsize[] = {
  1,1,1,1,2,
  2,4,4,4,4,
  8,4,1,1,1,
  1,1,1,
};

// local function
static int ievent = 0;

void
showEventTitle()
{
  std::cout << "------------------------ "
	    << "Event# "
	    << ievent << " "
	    << "--------------------------------"
	    << std::endl;
  ievent++;
}

// process_event routine
int
user_process_event( EVENT_HEADER* pheader, 
		    void* pevent, 
		    MidasBank& mbank )
{
  // 
  //std::cout << "user_process_event()" << std::endl;
  //
  // header
  showEventTitle();
  //
  printf("Evid:%04d- Mask:%04d- Serial:%d- Time:0x%08x Dsize:%d/0x%x \n",
	 pheader->event_id,
	 pheader->trigger_mask,
	 pheader->serial_number,
	 pheader->time_stamp,
	 pheader->data_size,
	 pheader->data_size);

  // get Bank List 
  std::list<BANK32> bankList = mbank.getBankDefs();
  //
  for( std::list<BANK32>::iterator i = bankList.begin();
       i != bankList.end(); i++ ){
    //
    std::cout << ""
              << " Bank " 
              << std::string( i->name,4 )
              << " type=" 
              << ( (0<i->type && i->type<18) ? tname[i->type] : "?" )
              << " Ndata="
              << ( (0<i->type && i->type<18) ? 
                   i->data_size / tsize[i->type] : i->data_size )
              << " size="
              << i->data_size
              << " (0x" << std::hex << i->data_size << ")" << std::dec
              << std::endl;
    if( options.Dumpevent ){
      DWORD* pdata = 0;
      char str[4];
      strncpy( str, i->name, 4);
      int nwords = bk_locate( pevent, str, &pdata );
      for( int i = 0; i < nwords; i++){
	if( i%9 == 0 ) std::cout << " -> ";
	std::cout << "0x" << std::hex << std::setw(8) << std::setfill('0')
		  << pdata[i] << " ";
	if( (i+1)%9 == 0 || i == nwords - 1)
	  std::cout << std::dec << std::endl;
      }


    }


  }
  std::cout << std::endl;

  return( 0 );
}


//////
// init routine
void
user_init()
{
  //
  //std::cout << "user_init()" << std::endl;
}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  //
  //std::cout << "user_begin_of_run()" << std::endl;
  // header
  showEventTitle();
  //

}

// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
  //std::cout << "user_end_of_run()" << std::endl;
  // header
  showEventTitle();
  //

}


// term routine
void
user_term()
{
  //
  //std::cout << "user_term()" << std::endl;
}
