/********************************************************************\

  Name:         analyzer.c
  Created by:   Stefan Ritt

  Contents:     System part of Analyzer code for sample experiment

  $Log: analyzer.cc,v $
  Revision 1.1.1.1  2002/08/13 12:48:12  kensh


  Start version.

  Revision 1.10  2002/05/10 05:22:34  pierre
  add MANA_LITE #ifdef

  Revision 1.9  2002/05/09 02:50:28  midas
  Removed initialization of 'Edit on start' by analyzer

  Revision 1.8  2002/05/08 19:54:40  midas
  Added extra parameter to function db_get_value()

  Revision 1.7  2000/11/20 12:29:37  midas
  Added use_tests flag in analyzer request

  Revision 1.6  2000/09/12 12:36:15  midas
  Removed test messages

  Revision 1.5  2000/08/11 11:43:50  midas
  Added cm_msg1 to produce messages which go to a differnt logging file

  Revision 1.4  2000/03/02 22:00:18  midas
  Changed events sent to double

  Revision 1.3  1998/10/29 14:18:19  midas
  Used hDB consistently

  Revision 1.2  1998/10/12 12:18:58  midas
  Added Log tag in header


\********************************************************************/
                                                        
/* standard includes */
#include <stdio.h>
#include <string.h>
#include <time.h>

/* midas includes */
#include "midas.h"

PAWC_DEFINE(1000000);

/*-- Globals -------------------------------------------------------*/

/* The analyzer name (client name) as seen by other MIDAS clients   */
char *analyzer_name = "EventChecker";

/* analyzer_loop is called with this interval in ms (0 to disable)  */
INT  analyzer_loop_period = 0;

/* default ODB size */
INT  odb_size = DEFAULT_ODB_SIZE;

/* ODB structures */
RUNINFO          runinfo;

/*-- Module declarations -------------------------------------------*/

extern ANA_MODULE event_checker_module;

ANA_MODULE *trigger_module[] = {
  &event_checker_module,
  NULL
};

/*-- Bank definitions ----------------------------------------------*/

BANK_LIST trigger_bank_list[] = {

  /* online banks */
  { "EVNT", TID_DWORD, 0, NULL },
  { "SMP0", TID_DWORD, 0, NULL },
  { "TDC0", TID_DWORD, 0, NULL },
  { "SMP1", TID_DWORD, 0, NULL },
  { "TDC1", TID_DWORD, 0, NULL },
  { "SCL0", TID_DWORD, 0, NULL },
  { "NGF0", TID_DWORD, 0, NULL },
  { "TRG0", TID_DWORD, 0, NULL },
  { "ADC0", TID_DWORD, 0, NULL },
  { "NGF1", TID_DWORD, 0, NULL },
  { "TRG1", TID_DWORD, 0, NULL },
  { "ADC1", TID_DWORD, 0, NULL },

  { "" },
};

/*-- Event request list --------------------------------------------*/

ANALYZE_REQUEST analyze_request[] = {
  { "Trigger",            /* equipment name */
    0x01,                 /* event ID */
    TRIGGER_ALL,          /* trigger mask */
    GET_ALL,             /* get some events */
    "SYSTEM",             /* event buffer */
    TRUE,                 /* enabled */
    "", "",               /* client, host */
    NULL,                 /* analyzer routine */
    trigger_module,       /* module list */
    trigger_bank_list,    /* bank list */
    1000,                 /* RWNT buffer size */
    TRUE,                 /* Use tests for this event */
  },

  { "" }
};

/*-- Analyzer Init -------------------------------------------------*/

INT analyzer_init()
{
  HNDLE hDB, hKey;
  char  str[80];
  
  RUNINFO_STR(runinfo_str);
  
  /* open ODB structures */
  cm_get_experiment_database(&hDB, NULL);
  db_create_record(hDB, 0, "/Runinfo", strcomb(runinfo_str));
  db_find_key(hDB, 0, "/Runinfo", &hKey);
  if (db_open_record(hDB, hKey, &runinfo, sizeof(runinfo), MODE_READ, NULL, NULL) != DB_SUCCESS){
    cm_msg(MERROR, "analyzer_init", "Cannot open \"/Runinfo\" tree in ODB");
    return 0;
  }
  
  // Do not send events to ODB, M.Yamaga
  sprintf( str, "/%s/Output/Events to ODB", analyzer_name );
  int  size = sizeof( BOOL );
  BOOL  var;
  if ( db_get_value( hDB,0, str, &var, &size, TID_BOOL, TRUE ) == DB_SUCCESS ){
    if ( var ){
      var = FALSE;
      db_set_value( hDB, 0, str, &var, sizeof(BOOL), 1, TID_BOOL );
      printf( "\n\'%s\' changed to %d\n\n",str,var );
    }
  }
  else{
    printf( "Cannot find \'%s\' key.\n",str );
  }

  // Do not clear histograms, M.Yamaga
  sprintf( str, "/%s/Output/Clear histos", analyzer_name );
  size = sizeof( BOOL );
  if ( db_get_value( hDB,0, str, &var, &size, TID_BOOL, TRUE ) == DB_SUCCESS ){
    if ( var ){
      var = FALSE;
      db_set_value( hDB, 0, str, &var, sizeof(BOOL), 1, TID_BOOL );
      printf( "\n\'%s\' changed to %d\n\n",str,var );
    }
  }
  else{
    printf( "Cannot find \'%s\' key.\n",str );
  }

  return SUCCESS;
}

/*-- Analyzer Exit -------------------------------------------------*/

INT analyzer_exit()
{
  return CM_SUCCESS;
}

/*-- Begin of Run --------------------------------------------------*/

INT ana_begin_of_run(INT run_number, char *error)
{
  return CM_SUCCESS;
}

/*-- End of Run ----------------------------------------------------*/

INT ana_end_of_run(INT run_number, char *error)
{
  FILE   *f;
  time_t now;
  char   str[256];
  int    size;
  double n;
  HNDLE  hDB;
  BOOL   flag;

  cm_get_experiment_database(&hDB, NULL);

  /* update run log if run was written and running online */

  size = sizeof(flag);
  db_get_value(hDB, 0, "/Logger/Write data", &flag, &size, TID_BOOL, TRUE);
  if (flag && runinfo.online_mode == 1){
    /* update run log */
    size = sizeof(str);
    str[0] = 0;
    db_get_value(hDB, 0, "/Logger/Data Dir", str, &size, TID_STRING, TRUE);
    if (str[0] != 0)
      if (str[strlen(str)-1] != DIR_SEPARATOR)
        strcat(str, DIR_SEPARATOR_STR);
    strcat(str, "runlog.txt");
    
    f = fopen(str, "a");
    
    time(&now);
    strcpy(str, ctime(&now));
    str[10] = 0;

    fprintf(f, "%s\t%3d\t", str, runinfo.run_number);

    strcpy(str, runinfo.start_time);
    str[19] = 0;
    fprintf(f, "%s\t", str+11);

    strcpy(str, ctime(&now));
    str[19] = 0;
    fprintf(f, "%s\t", str+11);

    size = sizeof(n);
    db_get_value(hDB, 0, "/Equipment/Trigger/Statistics/Events sent", &n, &size, TID_DOUBLE, TRUE);

    //fprintf(f, "%5.1lfk\t", n/1000);
    fprintf(f, "%.0f events\n", n);
    
    fclose(f);
  }
  
  return CM_SUCCESS;
}

/*-- Pause Run -----------------------------------------------------*/

INT ana_pause_run(INT run_number, char *error)
{
  return CM_SUCCESS;
}

/*-- Resume Run ----------------------------------------------------*/

INT ana_resume_run(INT run_number, char *error)
{
  return CM_SUCCESS;
}

/*-- Analyzer Loop -------------------------------------------------*/

INT analyzer_loop()
{
  return CM_SUCCESS;
}

/*------------------------------------------------------------------*/
