// -*- C++ -*-
//
// eventChecker.cc
// 
// Mon Jul  8 22:27:07 JST 2002
// M.Yamaga
//
// $Id: eventChecker.cc,v 1.1.1.1 2002/08/13 12:48:12 kensh Exp $
//
// $Log: eventChecker.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include <list>
#include <string.h>
#include <unistd.h>
#include "midas.h"
#include "unpack/TdcModule.h"
#include "unpack/TdcEvent.h"
#include "unpack/AdcModule.h"
#include "unpack/AdcEvent.h"

//
extern"C"{
INT event_checker( EVENT_HEADER*, void* );
INT event_checker_init( void );
INT event_checker_bor( INT run_number );
INT event_checker_eor( INT run_number );
}

//
ANA_MODULE event_checker_module = {
  "Event Checker",               /* module name           */  
  "Mitsuhiro YAMAGA",            /* author                */
  event_checker,                 /* event routine         */
  event_checker_bor,             /* BOR routine           */
  event_checker_eor,             /* EOR routine           */
  event_checker_init,            /* init routine          */
  NULL,                          /* exit routine          */
  NULL,                          /* parameter structure   */
  0,                             /* structure size        */
  NULL,                          /* initial parameters    */
};


//
extern RUNINFO  runinfo; // analyzer.cc


//
INT
event_checker( EVENT_HEADER* pheader, void* pevent )
{
  std::cout << "EventChecker:"
            << " Run#" << runinfo.run_number
            << " Serial#" << pheader->serial_number;

  // # of event in the buffer
  int  neSMP0,neSMP1,neNGF0,neNGF1;
  neSMP0 = neSMP1 = neNGF0 = neNGF1 = 0;

  DWORD*  pdataSMP0=0;
  int  ndataSMP0 = bk_locate( pevent, "SMP0", &pdataSMP0 );
  //std::cout << "pdataSMP=" << pdataSMP;
  if ( ndataSMP0 >= 3 && pdataSMP0 != 0 ){
    neSMP0 = pdataSMP0[1] & 0xffff;
  }

  DWORD*  pdataSMP1=0;
  int  ndataSMP1 = bk_locate( pevent, "SMP1", &pdataSMP1 );
  //std::cout << "pdataSMP=" << pdataSMP;
  if ( ndataSMP1 >= 3 && pdataSMP1 != 0 ){
    neSMP1 = pdataSMP1[1] & 0xffff;
  }
    
  DWORD*  pdataNGF0=0;
  int  ndataNGF0 = bk_locate( pevent, "NGF0", &pdataNGF0 );
  //std::cout << "pdataNGF0=" << pdataNGF0;
  if ( ndataNGF0 > 0 && pdataNGF0 != 0 ){
    neNGF0 = pdataNGF0[0];
  }
  
  DWORD*  pdataNGF1=0;
  int  ndataNGF1 = bk_locate( pevent, "NGF1", &pdataNGF1 );
  //std::cout << "pdataNGF1=" << pdataNGF1;
  if ( ndataNGF1 > 0 && pdataNGF1 != 0 ){
    neNGF1 = pdataNGF1[0];
  }

  // dump
  std::cout << " : Nevent :"
            << " SMP0=" << neSMP0 
            << " SMP1=" << neSMP1 
            << " NGF0=" << neNGF0 
            << " NGF1=" << neNGF1;
  
  if ( neSMP0 == neSMP1 && neSMP1 == neNGF0 && neNGF0 == neNGF1 ){
    std::cout << "   OK" << std::endl;
  }
  else{
    std::cout << " ## SLIP ##" << std::endl;
  }

  // EventNumber from TDC0 for the first event of the spill
  DWORD*  pdataTDC0 = 0;
  int  tdc0size = bk_locate( pevent, "TDC0", &pdataTDC0 );
  TdcEvent  tdc0( 0, pheader, pdataTDC0, tdc0size, 1 );
  int  tdc0_evnum = tdc0.tdc( 16 ).hitPattern() >> 56;

  // EventNumber from TDC1 for the first event of the spill
  DWORD*  pdataTDC1 = 0;
  int  tdc1size = bk_locate( pevent, "TDC1", &pdataTDC1 );
  TdcEvent  tdc1( 1, pheader, pdataTDC1, tdc1size, 1 );
  int  tdc1_evnum = tdc1.tdc( 15 ).hitPattern() >> 56;

  // EventNumber from ADC0 for the first event of the spill
  DWORD*  pdataADC0 = 0;
  int  adc0size = bk_locate( pevent, "ADC0", &pdataADC0 );
  AdcEvent  adc0( 0, pheader, pdataADC0, adc0size, 1 );
  int  adc0_evnum = adc0.adc( 18 ).hitPatternH(1000) >> 40;
  
  // EventNumber from ADC1 for the first event of the spill
  DWORD*  pdataADC1 = 0;
  int  adc1size = bk_locate( pevent, "ADC1", &pdataADC1 );
  AdcEvent  adc1( 1, pheader, pdataADC1, adc1size, 1 );
  int  adc1_evnum = adc1.adc( 18 ).hitPatternH(1000) >> 40;

  //
  std::cout << "              EventNumber :"
            << " TDC(0:16:56)="
            << std::hex << std::setw(2) << std::setfill('0')
            << tdc0_evnum
            << " TDC(1:15:56)="
            << std::hex << std::setw(2) << std::setfill('0')
            << tdc1_evnum
            << " ADC(0:18:88)="
            << std::hex << std::setw(2) << std::setfill('0')
            << adc0_evnum
            << " ADC(1:18:88)="
            << std::hex << std::setw(2) << std::setfill('0')
            << adc1_evnum
            << std::dec;

  //
  if ( tdc0_evnum == tdc1_evnum && 
       tdc0_evnum == adc0_evnum &&
       tdc0_evnum == adc1_evnum ){
    std::cout << "   OK" << std::endl;
  }
  else{
    std::cout << " ## SLIP ##" << std::endl;
  }

  return CM_SUCCESS;
}

//
INT
event_checker_init()
{
  
  return CM_SUCCESS;
}

//
INT
event_checker_bor( INT run_number )
{
  // message
  std::cout << "eventCheker : Run " << run_number
            << std::endl;

  return CM_SUCCESS;
}

//
INT
event_checker_eor( INT run_number )
{

  return CM_SUCCESS;
}

