// -*- C++ -*-
//
// moduleChecker.cc
// 
// Mon Jul  8 22:27:07 JST 2002
// M.Yamaga
//
// $Id: moduleChecker.cc,v 1.1.1.1 2002/08/13 12:48:12 kensh Exp $
//
// $Log: moduleChecker.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include <list>
#include <string.h>
#include <unistd.h>
#include "midas.h"
#include "unpack/TdcModule.h"
#include "unpack/TdcEvent.h"
#include "unpack/AdcModule.h"
#include "unpack/AdcEvent.h"

//
extern"C"{
INT module_checker( EVENT_HEADER*, void* );
INT module_checker_init( void );
INT module_checker_bor( INT run_number );
INT module_checker_eor( INT run_number );
}

//
ANA_MODULE module_checker_module = {
  "Module Checker",               /* module name           */  
  "Mitsuhiro YAMAGA",            /* author                */
  module_checker,                 /* event routine         */
  module_checker_bor,             /* BOR routine           */
  module_checker_eor,             /* EOR routine           */
  module_checker_init,            /* init routine          */
  NULL,                          /* exit routine          */
  NULL,                          /* parameter structure   */
  0,                             /* structure size        */
  NULL,                          /* initial parameters    */
};


//
extern RUNINFO  runinfo; // analyzer.cc

// 
static TdcEvent  oldTdc0;
static TdcEvent  oldTdc1;
static AdcEvent  oldAdc0;
static AdcEvent  oldAdc1;


//
INT
module_checker( EVENT_HEADER* pheader, void* pevent )
{
#if 0
  std::cout << "EventChecker2:"
            << " Run#" << runinfo.run_number
            << " Serial#" << pheader->serial_number
            << std::endl;
#endif

  // # of event in the buffer
  int  neSMP0,neSMP1,neNGF0,neNGF1;
  neSMP0 = neSMP1 = neNGF0 = neNGF1 = 0;

  DWORD*  pdataSMP0=0;
  int  ndataSMP0 = bk_locate( pevent, "SMP0", &pdataSMP0 );
  //std::cout << "pdataSMP=" << pdataSMP;
  if ( ndataSMP0 >= 3 && pdataSMP0 != 0 ){
    neSMP0 = pdataSMP0[1] & 0xffff;
  }

  DWORD*  pdataSMP1=0;
  int  ndataSMP1 = bk_locate( pevent, "SMP1", &pdataSMP1 );
  //std::cout << "pdataSMP=" << pdataSMP;
  if ( ndataSMP1 >= 3 && pdataSMP1 != 0 ){
    neSMP1 = pdataSMP1[1] & 0xffff;
  }
    
  DWORD*  pdataNGF0=0;
  int  ndataNGF0 = bk_locate( pevent, "NGF0", &pdataNGF0 );
  //std::cout << "pdataNGF0=" << pdataNGF0;
  if ( ndataNGF0 > 0 && pdataNGF0 != 0 ){
    neNGF0 = pdataNGF0[0];
  }
  
  DWORD*  pdataNGF1=0;
  int  ndataNGF1 = bk_locate( pevent, "NGF1", &pdataNGF1 );
  //std::cout << "pdataNGF1=" << pdataNGF1;
  if ( ndataNGF1 > 0 && pdataNGF1 != 0 ){
    neNGF1 = pdataNGF1[0];
  }

  // Check EventBuffer with previous data
  DWORD*  pdataTDC0 = 0;
  int  tdc0size = bk_locate( pevent, "TDC0", &pdataTDC0 );
  for( int i=1;i<=neSMP0;i++ ){
    TdcEvent  tdc0( 0, pheader, pdataTDC0, tdc0size, i );
    if ( tdc0.valid() ){
      if ( oldTdc0.valid() ){
        if ( tdc0.tdcList().size() == oldTdc0.tdcList().size() ){
          std::list<TdcModule>::const_iterator  mi;
          std::list<TdcModule>::const_iterator  mj;
          for( mi=tdc0.tdcList().begin(),  mj=oldTdc0.tdcList().begin();
               mi!=tdc0.tdcList().end() && mj!=oldTdc0.tdcList().end();
               mi++,mj++ ){
            bool  sameTdc0 = true;
            std::list<unsigned int>::const_iterator hi;
            std::list<unsigned int>::const_iterator hj;
            for( hi=mi->hitList().begin(),  hj=mj->hitList().begin();
                 hi!=mi->hitList().end() && hj!=mj->hitList().end();
                 hi++,hj++ ){
              if ( *hi != *hj ){
                sameTdc0 = false;
                break;
              }
            }
            if ( sameTdc0 ){
              std::cout << "TDC0 : same event found :"
                        << " serial=" << pheader->serial_number
                        << " event=" << i
                        << " ma=" << mi->ma()
                        << std::endl;
            }
          }
        }
      }
      oldTdc0 = tdc0;
    }
    else{ // No more event
      break;
    }
  }
  
  // Check EventBuffer with previous data
  DWORD*  pdataTDC1 = 0;
  int  tdc1size = bk_locate( pevent, "TDC1", &pdataTDC1 );
  for( int i=1;i<=neSMP0;i++ ){
    TdcEvent  tdc1( 0, pheader, pdataTDC1, tdc1size, i );
    if ( tdc1.valid() ){
      if ( oldTdc1.valid() ){
        if ( tdc1.tdcList().size() == oldTdc1.tdcList().size() ){
          std::list<TdcModule>::const_iterator  mi;
          std::list<TdcModule>::const_iterator  mj;
          for( mi=tdc1.tdcList().begin(),  mj=oldTdc1.tdcList().begin();
               mi!=tdc1.tdcList().end() && mj!=oldTdc1.tdcList().end();
               mi++,mj++ ){
            bool  sameTdc1 = true;
            std::list<unsigned int>::const_iterator hi;
            std::list<unsigned int>::const_iterator hj;
            for( hi=mi->hitList().begin(),  hj=mj->hitList().begin();
                 hi!=mi->hitList().end() && hj!=mj->hitList().end();
                 hi++,hj++ ){
              if ( *hi != *hj ){
                sameTdc1 = false;
                break;
              }
            }
            if ( sameTdc1 ){
              std::cout << "TDC1 : same event found :"
                        << " serial=" << pheader->serial_number
                        << " event=" << i
                        << " ma=" << mi->ma()
                        << std::endl;
            }
          }
        }
      }
      oldTdc1 = tdc1;
    }
    else{ // No more event
      break;
    }
  }
  
  // Check EventBuffer with previous data
  DWORD*  pdataADC0 = 0;
  int  adc0size = bk_locate( pevent, "ADC0", &pdataADC0 );
  for( int i=1;i<=neNGF0;i++ ){
    AdcEvent  adc0( 0, pheader, pdataADC0, adc0size, i );
    if ( adc0.valid() ){
      if ( oldAdc0.valid() ){
        if ( adc0.adcList().size() == oldAdc0.adcList().size() ){
          std::list<AdcModule>::const_iterator  mi;
          std::list<AdcModule>::const_iterator  mj;
          for( mi=adc0.adcList().begin(),  mj=oldAdc0.adcList().begin();
               mi!=adc0.adcList().end() && mj!=oldAdc0.adcList().end();
               mi++,mj++ ){
            bool  sameAdc0 = true;
            std::list<unsigned int>::const_iterator hi;
            std::list<unsigned int>::const_iterator hj;
            for( hi=mi->hitList().begin(),  hj=mj->hitList().begin();
                 hi!=mi->hitList().end() && hj!=mj->hitList().end();
                 hi++,hj++ ){
              if ( *hi != *hj ){
                sameAdc0 = false;
                break;
              }
            }
            if ( sameAdc0 ){
              std::cout << "ADC0 : same event found :"
                        << " serial=" << pheader->serial_number
                        << " event=" << i
                        << " ma=" << mi->pa()
                        << std::endl;
            }
          }
        }
      }
      oldAdc0 = adc0;
    }
    else{ // No more event
      break;
    }
  }

  // Check EventBuffer with previous data
  DWORD*  pdataADC1 = 0;
  int  adc1size = bk_locate( pevent, "ADC1", &pdataADC1 );
  for( int i=1;i<=neNGF1;i++ ){
    AdcEvent  adc1( 0, pheader, pdataADC1, adc1size, i );
    if ( adc1.valid() ){
      if ( oldAdc1.valid() ){
        if ( adc1.adcList().size() == oldAdc1.adcList().size() ){
          std::list<AdcModule>::const_iterator  mi;
          std::list<AdcModule>::const_iterator  mj;
          for( mi=adc1.adcList().begin(),  mj=oldAdc1.adcList().begin();
               mi!=adc1.adcList().end() && mj!=oldAdc1.adcList().end();
               mi++,mj++ ){
            bool  sameAdc1 = true;
            std::list<unsigned int>::const_iterator hi;
            std::list<unsigned int>::const_iterator hj;
            for( hi=mi->hitList().begin(),  hj=mj->hitList().begin();
                 hi!=mi->hitList().end() && hj!=mj->hitList().end();
                 hi++,hj++ ){
              if ( *hi != *hj ){
                sameAdc1 = false;
                break;
              }
            }
            if ( sameAdc1 ){
              std::cout << "ADC1 : same event found :"
                        << " serial=" << pheader->serial_number
                        << " event=" << i
                        << " ma=" << mi->pa()
                        << std::endl;
            }
          }
        }
      }
      oldAdc1 = adc1;
    }
    else{ // No more event
      break;
    }
  }
  
  return CM_SUCCESS;
  }

//
INT
module_checker_init()
{
  
  return CM_SUCCESS;
}

//
INT
module_checker_bor( INT run_number )
{
  // message
  std::cout << "moduleChecker : Run " << run_number
            << std::endl;

  return CM_SUCCESS;
}

//
INT
module_checker_eor( INT run_number )
{

  return CM_SUCCESS;
}

