// -*- C++ -*-
//
// user_ana.cc for pedestalFilter
//
// Author:  Mitsuhiro YAMAGA
// Created: Wed Aug 28 11:27:42 JST 2002
//
// $Id: user_ana.cc,v 1.2 2002/08/30 15:29:04 yamaga Exp $
//
// Revision History
//
// $Log: user_ana.cc,v $
// Revision 1.2  2002/08/30 15:29:04  yamaga
// test code removed.
//
// Revision 1.1  2002/08/29 14:31:18  yamaga
// Initial version.
//
//

#include <iostream>
#include <iomanip>
#include <list>
#include "unpack/AdcModule.h"
#include "unpack/AdcEvent.h"
#include "detparam/DetParam.h"
#include "midas.h"
#include "MidasManager/MidasManager.h"

//
extern int  Debug;

extern double  Nsigma;

//
DetParam&  detDB = DetParam::instance();
static int  Run_num = 0;

// init routine
void
user_init()
{
  // Histogram definition
}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  // 
  Run_num = run_number;

  //if ( !detDB.getDB( run_number ) ){
  if ( !detDB.getDB( 1 ) ){
    std::cout << "user_begin_of_run : detDB.getDB : "
              << "cannot open DB for Run " << run_number
              << std::endl;
  }
  if ( Debug )
    detDB.dumpDB();
}


// process_event routine
void
user_process_event( MidasFileIO& ifile, MidasFileIO& ofile )
{
  // ibank
  MidasBank*  ibank = ifile.bank();
  if ( Debug ){
    std::cout << "IBANK:" << std::endl;
    ibank->showBankDefs();
  }

  // show event#
  std::cout << "pedestalFilter :"
            << " Run#" << Run_num
            << " Serial#" << ibank->eventHeader()->serial_number
            << std::endl;
  
  // obank
  MidasBank*  obank = ofile.bank();
  obank->initFilter( ibank->eventHeader() );

  // loop over banks in ibank
  std::list<BANK32> bklist = ibank->getBankDefs();
  for( std::list<BANK32>::iterator i=bklist.begin();
       i!=bklist.end(); i++ ){
    // ADC0,ADC1
    if ( strncmp( i->name,"ADC0",4 ) == 0 || 
         strncmp( i->name,"ADC1",4 ) == 0 ){
      static const char*  adcname[] = { "ADC0","ADC1" };
      int  nadc = (strncmp( i->name,"ADC0",4 ) == 0 ) ? 0 : 1;
      // create ADC0 in obank
      DWORD*  pADC = (DWORD*)obank->createBank( adcname[nadc],TID_DWORD );
      // read ibank
      DWORD*  pdataADC;
      int  adcsize = ibank->getBank( adcname[nadc], &pdataADC );
      int  ct = 1;
      while( true ){
        AdcEvent  adcevent( nadc, ibank->eventHeader(), pdataADC, 
                            adcsize, ct );
        if ( ct == adcevent.serial() ){
          // loop over adc modules
          std::list<unsigned int> alist;
          for( std::list<AdcModule>::const_iterator 
                 m = adcevent.adcList().begin();
               m != adcevent.adcList().end(); m++ ){
            for( std::list<unsigned int>::const_iterator
                   h = m->hitList().begin();
                 h != m->hitList().end(); h++ ){
              //
              static const int  highrange = 1 << 23;
              int  ch  = (*h >> 16) & 0x7f;
              int  adc = *h & 0x0fff;
              //
              const E391_DETE_DB* adcdb = 
                detDB.db_adc( m->crate(),m->pa(),ch );
              if ( adcdb ){
                if ( !(*h & highrange) && 
                     (adc-(adcdb->pedestalL+adcdb->pedWidthL*Nsigma)>= 0)){
                  alist.push_back( *h );
                  if ( Debug ){
                    std::cout << "ADC("
                              << m->crate() << "," 
                              << m->pa() << ","
                              << ch << ")"
                              << " : L : " << adc 
                              << " : " << adcdb->pedestalL
                              << std::endl;
                  }
                }
                else if( (*h & highrange) && 
                         (adc-(adcdb->pedestalH+adcdb->pedWidthH*Nsigma)>= 0)){
                  alist.push_back( *h );
                  if ( Debug ){
                    std::cout << "ADC("
                              << m->crate() << "," 
                              << m->pa() << ","
                              << ch << ")"
                              << " : H : " << adc 
                              << " : " << adcdb->pedestalH
                              << std::endl;
                  }
                }
              }
              else{
                alist.push_back( *h );
                if ( Debug ){
                  std::cout << "Cannot find DB for ADC(" 
                            << m->crate() << "," 
                            << m->pa() << ","
                            << ch << ")"
                            << std::endl;
                }
              }
            }
          }
          // Fill ADCx in obank
          *pADC++ = (ct << 16) | (alist.size() + 1);
          for( std::list<unsigned int>::iterator h = alist.begin();
               h != alist.end(); h++ ){
            *pADC++ = *h;
          }
          *pADC++ = 0xff000000;
          //
          ct++;
        }
        else{
          break; // no more ADCx event
        }
      }
      if ( ct > 1 ){
        *pADC++ = 0xffffffff;
      }
      obank->closeBank( pADC );
      if ( Debug ){
        std::cout << adcname[nadc] << " : " << ct - 1 << " events processed"
                  << std::endl;
      }
    }
    else{
      obank->copyBank( ibank, std::string(i->name,4) );
    }
  }
  
  // dump obank
  if ( Debug ){
    std::cout << "OBANK:" << std::endl;
    obank->showBankDefs();
  }

  // save to file
  ofile.writeEvent();
}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
}


// term routine
void
user_term()
{
  //
}


