// -*- C++ -*-
//
// ngf_1885n.cc
// utility
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Mon Apr  8 21:14:27 JST 2002
//
// $Id: ngf_1885n.cc,v 1.2 2002/08/29 12:52:07 yamaga Exp $
//
// Revision History
//
// $Log: ngf_1885n.cc,v $
// Revision 1.2  2002/08/29 12:52:07  yamaga
// Block Transfer mode added. Do not dump to ODB.
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "midas.h"
#include "mcstd.h"

#include "ngf/Ngf.h"
#include "fbcrate/FBcrate.h"
#include "vmeioreg/VmeIoreg.h"
#include "ngf_1885n.h"

//
static int    debugLevel = 0;

// START flag
static bool  Started = false;

// POLLING
bool    PollTrigger = false;

// For 1885N : High-range or Auto range
#define ADCRANGE_AUTO     0x0
#define ADCRANGE_LOW      0x1
#define ADCRANGE_HIGH     0x2
#define ADCRANGE_SWITCH   0x4
#define ADCRANGE_REVERSE  0x8
#define LRS1885N_AUTORANGE  0x00001000
#define LRS1885N_HIGHRANGE  0x10002000
#define LRS1885N_LOWRANGE   0x30000000
static int  AdcRange        = ADCRANGE_AUTO;
static int  RangePeriod1    = 0; // #spills. For MIX mode
static int  RangePeriod2    = 0; // #spills. For MIX mode

static int  CurRange  = 0;
static int  CurPeriod = 0;
static bool ChangeAdcRangeFlag = false;

// NGF
static const unsigned int  NGF_BASEADDR = 0x80e00000;
Ngf  ngf( NGF_BASEADDR );

// FASTBUS crate and modules
FBcrate  fbcrate;

// Data buffer for module
typedef struct{
  int            pa;
  int            data_size;
  unsigned int*  buffer;
  unsigned int*  dmaptr;
} MODULEDATABUF;
static MODULEDATABUF   lrs1885n_template;
static MODULEDATABUF*  module_template;
static int  Nmodules = 0; // modules other than 1885N

// Data buffer for event
typedef struct{
  int             event_number;
  unsigned int    trigger;
  unsigned int    time;
  unsigned int    status[8];
  MODULEDATABUF   lrs1885n;
  MODULEDATABUF*  modulebuf;
} EVENTDATABUF;
static EVENTDATABUF*  databuf;

static int  Nevent  = 0;
static int  Nstatus = 0;
static int  MaxEvent = 1000; // dummy
static bool Overflow = false;

// VME-IOREG
static const unsigned int  VIOREG_BASEADDR  = 0x8000;
VmeIoreg  vioreg( VIOREG_BASEADDR );


////////////////
// Interrupt Handler
bool  Interrupted = false;
extern"C"
void
sighdl( int arg )
{
  Interrupted = true;
  
  if ( debugLevel ){
    std::cerr << "Interrupt received. arg=" << arg << std::endl;
  }
}


// Required functions for FBcrate class
void
frc( unsigned int pa,unsigned int sa,unsigned int* data )
{
  ngf.frc( pa,sa,data );
}

void
fwc( unsigned int pa,unsigned int sa,unsigned int data )
{
  ngf.fwc( pa,sa,data );
}

void
frd( unsigned int pa,unsigned int sa,unsigned int* data )
{
  ngf.frd( pa,sa,data );
}

void
fwd( unsigned int pa,unsigned int sa,unsigned int data )
{
  ngf.fwd( pa,sa,data );
}

void
reset()
{
  ngf.SeqResetReg( 0 );
  ngf.SeqEnableReg( 0 );
}


////////////////
void
initSettings()
{
  // set special keys
  int  size = sizeof(INT);
  HNDLE  hDB;
  cm_get_experiment_database(&hDB, NULL);
  db_create_record( hDB, 0, "/Equipment/NGF1-ADC/Settings",
                    "AdcRange = INT : 0\n"
                    "RangePeriod1 = INT : 3\n"
                    "RangePeriod2 = INT : 897"
                    );
}


////////////////
static void
getAdcRange()
{
  int  size = sizeof(INT);
  HNDLE  hDB;
  cm_get_experiment_database(&hDB, NULL);
  //
  if ( db_get_value( hDB,0, "/Equipment/NGF1-ADC/Settings/AdcRange", 
                     &AdcRange, &size, TID_INT, TRUE) == DB_SUCCESS  ){
    int  sw  = AdcRange & ADCRANGE_SWITCH;
    int  rev = AdcRange & ADCRANGE_REVERSE;
    int  rg  = AdcRange & 0x03;
    if ( !sw ){
      if ( rg == ADCRANGE_AUTO )
        std::cerr << std::endl << "ADC range is Auto          " << std::endl;
      else if ( rg == ADCRANGE_HIGH )
        std::cerr << std::endl << "ADC range is High          " << std::endl;
      else if ( rg == ADCRANGE_LOW )
        std::cerr << std::endl << "ADC range is Low           " << std::endl;
      else
        std::cerr << std::endl << "ADC range is ?(Auto)       " << std::endl;
    }
    else if ( !rev ){
      if ( rg == ADCRANGE_AUTO ){
        std::cerr << std::endl << "ADC range is ?(Auto)       " << std::endl;
        sw = 0;
      }
      else if ( rg == ADCRANGE_HIGH )
        std::cerr << std::endl << "ADC range is High+Auto     " << std::endl;
      else if ( rg == ADCRANGE_LOW )
        std::cerr << std::endl << "ADC range is Low+Auto      " << std::endl;
      else if ( rg == (ADCRANGE_HIGH|ADCRANGE_LOW) )
        std::cerr << std::endl << "ADC range is High+Low      " << std::endl;
    }
    else { // sw + rev
      if ( rg == ADCRANGE_AUTO ){
        std::cerr << std::endl << "ADC range is ?(Auto)       " << std::endl;
        sw = 0;
      }
      else if ( rg == ADCRANGE_HIGH )
        std::cerr << std::endl << "ADC range is Auto+High     " << std::endl;
      else if ( rg == ADCRANGE_LOW )
        std::cerr << std::endl << "ADC range is Auto+Low      " << std::endl;
      else if ( rg == (ADCRANGE_HIGH|ADCRANGE_LOW) )
        std::cerr << std::endl << "ADC range is Low+High      " << std::endl;
    }
  }
  else{
    std::cerr << "Cannot find AdcRange key." << std::endl;
  }
  
  //
  if ( AdcRange & ADCRANGE_SWITCH ){
    if ( db_get_value( hDB,0, "/Equipment/NGF1-ADC/Settings/RangePeriod1", 
                       &RangePeriod1, &size, TID_INT, TRUE) == DB_SUCCESS ){
      std::cerr << "RangePeriod1 is " << RangePeriod1 << " spill(s)   " 
                << std::endl;
    }
    else{
      std::cerr << "Cannot find RangePeriod1 key." << std::endl;
    }
    
    if ( db_get_value( hDB,0, "/Equipment/NGF1-ADC/Settings/RangePeriod2", 
                       &RangePeriod2, &size, TID_INT, TRUE) == DB_SUCCESS ){
      std::cerr << "RangePeriod2 is " << RangePeriod2 << " spill(s)   " 
                << std::endl;
    }
    else{
      std::cerr << "Cannot find RangePeriod2 key." << std::endl;
    }
    
  }
}
  

////////////////
static void
changeAdcRange()
{
  int  sw  = AdcRange & ADCRANGE_SWITCH;
  int  rev = AdcRange & ADCRANGE_REVERSE;
  int  rg  = AdcRange & 0x03;

  unsigned int  rangebit;
    
  if ( rg == ADCRANGE_HIGH ){ // high-auto or auto-high
    if ( CurRange == ADCRANGE_AUTO || CurRange == -1 ){
      rangebit  = LRS1885N_HIGHRANGE;
      CurRange  = ADCRANGE_HIGH;
      CurPeriod = (!rev) ? RangePeriod1 : RangePeriod2;
    }
    else{
      rangebit  = LRS1885N_AUTORANGE;
      CurRange  = ADCRANGE_AUTO;
      CurPeriod = (!rev) ? RangePeriod2 : RangePeriod1;
    }
  }
  else if ( rg == ADCRANGE_LOW ){ // low-auto or auto-low
    if ( CurRange == ADCRANGE_AUTO || CurRange == -1 ){
      rangebit  = LRS1885N_LOWRANGE;
      CurRange  = ADCRANGE_LOW;
      CurPeriod = (!rev) ? RangePeriod1 : RangePeriod2;
    }
    else{
      rangebit  = LRS1885N_AUTORANGE;
      CurRange  = ADCRANGE_AUTO;
      CurPeriod = (!rev) ? RangePeriod2 : RangePeriod1;
    }
  }
  else if ( rg == (ADCRANGE_HIGH|ADCRANGE_LOW) ){ // high-low or low-high
    if ( sw ){
      if ( CurRange == ADCRANGE_LOW || CurRange == -1 ){
        rangebit  = LRS1885N_HIGHRANGE;
        CurRange  = ADCRANGE_HIGH;
        CurPeriod = (!rev) ? RangePeriod1 : RangePeriod2;
      }
      else{
        rangebit  = LRS1885N_LOWRANGE;
        CurRange  = ADCRANGE_LOW;
        CurPeriod = (!rev) ? RangePeriod2 : RangePeriod1;
      }
    }
    else{
      if ( CurRange == -1 ){
        rangebit  = LRS1885N_AUTORANGE;
        CurRange  = ADCRANGE_AUTO;
        CurPeriod = (!rev) ? RangePeriod1 : RangePeriod2;
      }
    }
  }
    
  //
  ngf.disableList();
  for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
       s != fbcrate.slotList().end(); s++ ){
    if ( s->name() == "LRS1885N" ){
      ngf.fwc( s->pa(),0,rangebit );
#if 0
      unsigned int  data;
      ngf.frc( s->pa(),0,&data );
      std::cout << std::hex << data << std::dec << std::endl;
      std::cout << std::hex << ngf.SeqStatusReg() << std::dec << std::endl;
#endif        
    }
  }
  ngf.enableList();

#if 0
  std::cout << "ADCrange=" << std::hex << CurRange << std::dec << std::endl;
#endif

  ChangeAdcRangeFlag = false;
}


////////////////
void
initNgf()
{
  if ( debugLevel )
    std::cerr << "initNgf" << std::endl;

  // Use Interrupt for trigger event?
  if ( getenv( "POLL_TRIGGER" ) ){
    PollTrigger = true;
    std::cout << std::endl
              << "POLL_TRIGGER environment variable is set."
              << std::endl;
  }
  else{
    PollTrigger = false;
    std::cout << std::endl
              << "POLL_TRIGGER environment variable is NOT set."
              << std::endl;
  }
  
  // Disable interrupt
  vioreg.disableInt();
  vioreg.clear();
  vioreg.mask();
  vioreg.level_out( 0x00 );
  
  // VME interrupt setup
  if ( !VmeIoreg::initVuiInterrupt( 0,5,0x50 ) ){
    abort();
  }
  sigset( SIGINT,sighdl );

  // NGF
  ngf.SeqResetReg( 0 );
  ngf.FbTimeoutReg( 0x73 );
  ngf.SeqEnableReg( 0 );
  ngf.maskInterrupt( NGF_CLEAR_INT_ALL ); // disable all

  // FBmodule
  fbcrate.readConf( "FBcrate.conf" );
  fbcrate.setFunc( frc,fwc,frd,fwd,reset );
  fbcrate.init();

}

  
////////////////
void
initNgfCommandList()
{
  if ( debugLevel )
    std::cerr << "initNgfCommandList" << std::endl;
  
  // Set NGF list command
  std::cout << "Loading command list : ";
  std::list<NgfList>  ls;
  ls.push_back( NgfList( NGF_WAIT_SEQ_GO_FLAG,0 ) );
  ls.push_back( NgfList( NGF_SET_SEQUENCER_OUT,NGF_OUT_NIM1 ) );
  
  // For modules other than 1885N : Block Transfer
  unsigned int   blt_buf     = ngf.slaveAddr();
  unsigned int   blt_mode    = 0x0a; // (VME | BLT32), see page31
  unsigned int   blt_limitct = 0x00ffffff;
  unsigned int   blt_maxwd   = (blt_mode << 24) | (blt_limitct);
  for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
       s != fbcrate.slotList().end(); s++ ){
    if ( s->valid() && s->name() != "LRS1885N" ){
      // Block Transfer
      ls.push_back( NgfList( FB_PRIM_DSR,      s->pa()   ) );
      ls.push_back( NgfList( FB_SECAD_W,       0         ) );
      ls.push_back( NgfList( FB_LD_BLT_AP,     blt_buf   ) );
      ls.push_back( NgfList( FB_START_FRDB,    blt_maxwd ) );
      ls.push_back( NgfList( FB_DISCON,        0         ) );
      ls.push_back( NgfList( FB_STORE_FRDB_WC, 0         ) );
      ls.push_back( NgfList( FB_STORE_FRDB_AP, 0         ) );
      //std::cout << "BLT:buf=" << std::hex << blt_buf
      //          << " blt_maxwd=" << blt_maxwd
      //          << std::dec << std::endl;
      blt_buf += s->channelList().size()*sizeof(int);
    }
  }
  
  // For 1885n : channel by channel reading
  for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
       s != fbcrate.slotList().end(); s++ ){
    if ( s->valid() && s->name() == "LRS1885N" ){
      ls.push_back( NgfList( FB_PRIM_DSR,s->pa() ) );
      for( std::list<int>::const_iterator sa = s->channelList().begin();
           sa != s->channelList().end(); sa++ ){
        //ls.push_back( NgfList( FB_SECAD_W,*sa ) );
        ls.push_back( NgfList( FB_DATA_R,0 ) );
      }
      ls.push_back( NgfList( FB_DISCON,0 ) );
    }
  }

  // End of event
  ls.push_back( NgfList( NGF_SET_SEQUENCER_OUT,
                         NGF_CLEAR_NIM1 | NGF_OUT_NIM2 | NGF_OUT_NIM3 ) );
  ls.push_back( NgfList( NGF_SET_SEQUENCER_OUT,
                         NGF_CLEAR_NIM2 | NGF_CLEAR_NIM3 ) );
  ls.push_back( NgfList( NGF_CLEAR_SEQ_GO_FLAG,0 ) );
  ls.push_back( NgfList( NGF_ENABLE_RAM_SEQUENCER,0 ) );
  //
  ngf.setList( 0x0000, ls );
  //
  std::cout << ls.size() << " sequence." << std::endl;

  ////////
  // Data size per event
  int  DataSize = 0;
  for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
       s != fbcrate.slotList().end(); s++ ){
    if ( s->valid() ){
      if ( s->name() == "LRS1877S" ){
        DataSize += s->channelList().size()/4; // estimation
      }
      else{
        DataSize += s->channelList().size();
      }
    }
  }
  std::cout << "Data size = max " << DataSize << " channels/Event." 
            << std::endl;
  
  // Max event number per spill
  extern INT  max_event_size; // in frontend.cc, fe2.cc
  int  evsize = 1 + DataSize + 1 + 1; //header,terminator
  MaxEvent = max_event_size / sizeof(int) / evsize;
  std::cout << "MaxEvent = " << MaxEvent << "/spill." << std::endl;

  // Event buffer
  databuf = new EVENTDATABUF[MaxEvent];
  if ( debugLevel ){
    std::cout << "databuf : " 
              << databuf 
              << " : " << sizeof(databuf) 
              << " x " << MaxEvent
              << std::endl;
  }
  if ( databuf == NULL ){
    perror( "Cannot allocate databuf" );
    abort();
  }

  // 1885n : channel by channel reading
  int  dsize_1885n = 0;
  Nmodules = 0;
  for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
       s != fbcrate.slotList().end(); s++ ){
    if ( s->valid() ){
      if ( s->name() == "LRS1885N" )
        dsize_1885n += s->channelList().size();
      else
        Nmodules++; // # of modules other than 1885N
    }
  }
  if ( dsize_1885n > 0 ){
    // template
    lrs1885n_template.pa = 0;
    lrs1885n_template.data_size = dsize_1885n;
    for( int i=0;i<MaxEvent;i++ ){
      databuf[i].lrs1885n.buffer = new unsigned int[dsize_1885n];
      if ( debugLevel ){
        std::cout << "databuf[].lrs1885n.buffer : " 
                  << databuf[i].lrs1885n.buffer  
                  << " : " << sizeof(databuf[i].lrs1885n.buffer)
                  << " x " << dsize_1885n
                  << std::endl;
      }
      if ( databuf[i].lrs1885n.buffer == NULL ){
        perror( "Cannot allocate modulebuf.buffer." );
        abort();
      }
    }
  }

  // Other modules : block transfer
  if ( Nmodules > 0 ){
    // template
    module_template = new MODULEDATABUF[Nmodules];
    if ( debugLevel ){
      std::cout << "module_template : "
                << module_template
                << " : " << sizeof(module_template)
                << " x " << Nmodules
                << std::endl;
    }
    if ( module_template == NULL ){
      perror( "Cannot allocate module_template." );
      abort();
    }
    //
    int  ct = 0;
    unsigned int*  dmaptr = (unsigned int*)ngf.slavePtr();
    for( std::list<FBslot>::const_iterator s = fbcrate.slotList().begin();
         s != fbcrate.slotList().end(); s++ ){
      if ( s->valid() && s->name() != "LRS1885N" ){
        module_template[ct].pa        = s->pa();
        module_template[ct].data_size = s->channelList().size();
        module_template[ct].dmaptr    = dmaptr;
        dmaptr += module_template[ct].data_size;
        if ( debugLevel ){
          std::cout << "mod" << ct << " dmaptr=" << module_template[ct].dmaptr
                    << std::endl;
        }
        ct++;
      }
    }
    
    for( int i=0;i<MaxEvent;i++ ){
      databuf[i].modulebuf = new MODULEDATABUF[Nmodules];
      if ( debugLevel ){
        std::cout << "databuf[].modulebuf : "
                  << databuf[i].modulebuf
                  << " : " << sizeof(databuf[i].modulebuf)
                  << " x " << Nmodules
                  << std::endl;
      }
      if ( databuf[i].modulebuf == NULL ){
        perror( "Cannot allocate databuf.modulebuf" );
        abort();
      }
    }

    for( int i=0;i<MaxEvent;i++ ){
      for( int m=0;m<Nmodules;m++ ){
        databuf[i].modulebuf[m].buffer    
          = new unsigned int[module_template[m].data_size];
        if ( debugLevel ){
          std::cout << "databuf[].modulebuf[].buffer : "
                    << databuf[i].modulebuf[m].buffer
                    << " : " << sizeof(databuf[i].modulebuf[m].buffer)
                    << " x " << module_template[m].data_size
                    << std::endl;
        }
        if ( databuf[i].modulebuf[m].buffer == NULL ){
          perror( "Cannot allocate modulebuf" );
          abort();
        }
      }
    }
  }

}


////////////////
void
termNgf()
{
  if ( debugLevel )
    std::cerr << "termNgf" << std::endl;

  ngf.disableList();
  
  vioreg.disableInt();
  
  // close
  VmeIoreg::termVuiInterrupt();
  sigset( SIGINT,SIG_DFL );

  //
  for( int i=0;i<MaxEvent;i++ ){
    delete databuf[i].lrs1885n.buffer;
    for( int m=0;m<Nmodules;m++ ){
      delete databuf[i].modulebuf[m].buffer;
    }
    delete [] databuf[i].modulebuf;
  }
  delete [] databuf;
  delete [] module_template;
}


////////////////
void
beginRunNgf()
{
  if ( debugLevel )
    std::cerr << "beginRunNgf" << std::endl;
  
  // For 1885N : High-range or Auto range
  getAdcRange();
  CurRange  = -1;
  changeAdcRange();
  if ( AdcRange & ADCRANGE_REVERSE )
    changeAdcRange(); // again
  CurPeriod++; // for dummy spill
#if 0
  std::cout << "ADCrange=" << std::hex << CurRange << std::dec << std::endl;
#endif
  
  // Enable NGF list sequencer
  ngf.enableList();
  
  // Wait until all frontends are ready.
  Started = false;
  vioreg.clear();
  vioreg.level_out( 0x80 );
  //std::cerr << "Waiting other frontends ready." << std::endl;

  // Enable interrupt
  Nevent = 0;
  Nstatus = 0;
  Overflow = false;
  Interrupted = false;
  if ( PollTrigger ){
    vioreg.enableInt(1);
  }
  else{
    vioreg.enableInt(1);
    vioreg.enableInt(2);
  }

}


////////////////
void
endRunNgf()
{
  if ( debugLevel )
    std::cerr << "endRunNgf" << std::endl;

  // Disable NGF list sequencer
  ngf.disableList();

  // Disable interrupt
  vioreg.disableInt();
  vioreg.level_out( 0x00 );

  // stop
  Started = false;
}


////////////////
void
pauseRunNgf()
{
  if ( debugLevel )
    std::cerr << "pauseRunNgf" << std::endl;
 
  // Disable NGF list sequencer
  ngf.disableList();
  
  // Disable interrupt
  vioreg.disableInt();
  vioreg.level_out( 0x00 );

  // stop
  Started = false;
}


////////////////
void
resumeRunNgf()
{
  if ( debugLevel )
    std::cerr << "resumeRunNgf" << std::endl;
 
  // Enable NGF list sequencer
  ngf.enableList();
  
  // Wait until all frontends are ready.
  Started = false;
  vioreg.clear();
  vioreg.level_out( 0x80 );

  // Enable interrupt
  Nevent = 0;
  Nstatus = 0;
  Overflow = false;
  Interrupted = false;
  if ( PollTrigger ){
    vioreg.enableInt(1);
  }
  else{
    vioreg.enableInt(1);
    vioreg.enableInt(2);
  }

}


//#define CLEARFIRST

////////////////
void
bufferEventNgf()
{
  if ( debugLevel )
    std::cerr << "bufferEventNgf" << std::endl;
  
  // Start reading from FIFO.
  // Event time
  unsigned int  eventtime = time(NULL);
  // read trigger info
  unsigned int  trig = vioreg.input_flipflop() & 0xff;
  if ( CurRange == ADCRANGE_HIGH )
    trig |= 0x8000;
  else if ( CurRange == ADCRANGE_LOW )
    trig |= 0x4000;
#ifdef CLEARFIRST
  // AdcRange
  if ( ChangeAdcRangeFlag )
    changeAdcRange();
  // accept next event
  Interrupted = false;
  vioreg.clear(2);// Trigger signal
  vioreg.clear(3);// Trigger info
  vioreg.out( 0x0f );
#endif
  ngf.VmeOutReg( NGF_OUT_NIM3 );
  // NGF status
  Nstatus = 0;
  
  // read each trigger event
  if ( Nevent < MaxEvent ){
    databuf[Nevent].event_number = Nevent;
    databuf[Nevent].trigger = trig;
    databuf[Nevent].time = eventtime;
    databuf[Nevent].status[Nstatus++] = ngf.SeqFifoAndEclNimInputReg();
    databuf[Nevent].status[Nstatus++] = ngf.FbStatusReg1();
    databuf[Nevent].status[Nstatus++] = ngf.FbStatusReg2();
    //
    //std::cerr << "TRG=" << std::hex << databuf[Nevent].trigger 
    //          << std::dec << "\n";

    // Modules other than 1885N
    if ( Nmodules > 0 ){
      unsigned int  bltstat[32*2];
      int  stat = ngf.readFifoNoWait( 2*Nmodules, bltstat );
      for( int m=0;m<Nmodules;m++ ){
        unsigned int  wc   = bltstat[m*2] & 0x00ffffff;
        unsigned int  next = bltstat[m*2+1];
        databuf[Nevent].modulebuf[m].data_size = wc;
        if ( debugLevel ){
          std::cout << "mod=0x" << std::hex << m 
                    << " next=" << next 
                    << " WC=" << wc << std::dec << std::endl;
        }
        if ( wc > 0 ){
          memcpy( databuf[Nevent].modulebuf[m].buffer,
                  module_template[m].dmaptr,
                  wc*sizeof(unsigned int) );
        }
        if ( debugLevel ){
          for( int j=0;j<wc;j++ ){
            std::cout << "ch" << j << ":" << std::hex 
                      << module_template[m].dmaptr[j] << std::endl;
          }
        }
      }
    }

    // 1885N
    int  data_size = lrs1885n_template.data_size;
    databuf[Nevent].lrs1885n.data_size = data_size;
    if ( data_size > 0 ){
      while( 1 ){
        //int  stat = ngf.readFifo(data_size, databuf[Nevent].lrs1885n.buffer);
        int  stat = ngf.readFifoNoWait( data_size,
                                        databuf[Nevent].lrs1885n.buffer );
        if ( debugLevel )
          std::cout << "bufferEventNgf : stat=0x" 
                    << std::setw(8) << std::setfill('0') << std::hex
                    << stat
                    << std::dec << std::setw(0) << std::endl;
        if ( stat & 0xf0 ){ // sequence failed due to module busy
          std::cerr << "\rbufferEventNgf : module busy : SeqStat=0x" 
                    << std::setw(8) << std::setfill('0') << std::hex
                    << stat
                    << " FBStat=0x" 
                    << ngf.FbStatusReg1()
                    << ",0x" 
                    << ngf.FbStatusReg2()
                    << std::dec << std::setw(0) << std::endl;
          ngf.enableList();
          std::cerr << "bufferEventNgf : Ngf::enableList called" << std::endl;
        }
        else{
          break;
        }
      }
    }
    //
    databuf[Nevent].status[Nstatus++] = ngf.SeqStatusReg();
    //
    Nevent++;
  }
  else{
    if ( !Overflow ){
      std::cerr << "\rbufferEventNgf : Nevent exceeds the max size "
                << MaxEvent << std::endl;
      Overflow = true;
    }
  }

  // End of reading from FIFO.
  ngf.VmeOutReg( NGF_CLEAR_NIM3 );

#ifndef CLEARFIRST
  // AdcRange
  if ( ChangeAdcRangeFlag )
    changeAdcRange();
  // accept next event
  Interrupted = false;
  vioreg.clear(2);// Trigger signal
  vioreg.clear(3);// Trigger info
  vioreg.out( 0x0f );
#endif
}


////////////////
void
fillBankNgf( char* pevent )
{
  if ( debugLevel )
    std::cerr << "fillBankNgf" << std::endl;
  
  // Only in the start sequence.
  if ( !Started ){
    vioreg.level_out( 0x00 );
    vioreg.out( 0x01 );
    Started = true;
    //
    return;
  }
  
  // create NFG0 bank
  DWORD  *pdata;
  bk_create(pevent, "NGF0", TID_DWORD, &pdata);
  *pdata++ = Nevent | (Overflow ? (0x01 << 31) : 0 );
  *pdata++ = Nstatus;
  for( int i=0; i<Nevent; i++ ){
    for( int j=0; j<Nstatus; j++ ){
      *pdata++ = databuf[i].status[j];
    }
  }
  bk_close(pevent, pdata);

  // create TRG1 bank
  bk_create(pevent, "TRG0", TID_DWORD, &pdata);
  for( int i=0; i<Nevent; i++ ){
    *pdata++ = (((databuf[i].event_number+1)<<16) | databuf[i].trigger);
  }
  bk_close(pevent, pdata);
  
  // create ADC bank
  bk_create(pevent, "ADC0", TID_DWORD, &pdata);

  for( int i=0; i<Nevent; i++ ){
    int  dsize = databuf[i].lrs1885n.data_size;
    *pdata++ = ( ( (databuf[i].event_number+1) << 16 ) | (dsize + 1) );
    for( int j=0; j<dsize; j++ ){
    //for( int j=0; j<6; j++ ){
      *pdata++ = databuf[i].lrs1885n.buffer[j];
    }
    *pdata++ = 0xff000000;
  }
  if ( Nevent > 0 && lrs1885n_template.data_size > 0 )
    *pdata++ = 0xffffffff;

  // close ADC bank
  bk_close(pevent, pdata);

  // MSC0 bank
  if ( Nmodules > 0 ){
    bk_create(pevent, "MSC0", TID_DWORD, &pdata);

    for( int i=0; i<Nevent; i++ ){
      for( int m=0; m<Nmodules; m++ ){
        int  dsize = databuf[i].modulebuf[m].data_size;
        *pdata++ = ( ( (databuf[i].event_number+1) << 16 ) | ( dsize + 1 ) );
        for( int j=0; j<dsize; j++ ){
          //for( int j=0; j<6; j++ ){
          *pdata++ = databuf[i].modulebuf[m].buffer[j];
        }
        *pdata++ = 0xff000000;
      }
    }
    if ( Nevent > 0 )
      *pdata++ = 0xffffffff;
    
    bk_close(pevent, pdata);
  }

  // ADC Range
  if ( AdcRange & ADCRANGE_SWITCH && --CurPeriod <= 0 )
    ChangeAdcRangeFlag = true;

  // clear buffer for next event
  Nevent = 0;
  Overflow = false;
  Interrupted = false;
  vioreg.clear(1);// Spill signal

  ngf.VmeOutReg( NGF_OUT_NIM4 );
  ngf.VmeOutReg( NGF_CLEAR_NIM4 );
}

