// -*- C++ -*-
//
// Smp.h
// SMP-TKO accessing class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Wed Nov 21 13:14:21 JST 2001
//
// $Id: Smp.h,v 1.1.1.1 2002/08/13 12:48:12 kensh Exp $
//
// Revision History
//
// $Log: Smp.h,v $
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#ifndef  SMP_H_INCLUDED
#define  SMP_H_INCLUDED

// includes
#include <sys/vme.h>


//////
#define  SMP_BUFSIZE    (256*1024) // 256kword x 4byte/word = 1MB

#define  SMP_CMR        0x00180000  // command register
#define  SMP_STAR       0x00180000  // status register
#define  SMP_SNCCR      0x00180004  // serial number counter control register
#define  SMP_SNCR       0x00180008  // serial number counter register
#define  SMP_DSR        0x0018000C  // data size register
#define  SMP_ENR        0x00180010  // event number register
#define  SMP_BCR        0x00180014  // buffer control register
#define  SMP_SCR        0x00180018  // Switch Condition Register
#define  SMP_ICR        0x0018001C  // Interrupt Control Register
#define  SMP_IVR        0x00180020  // Interrupt Vector Register

#define  SMP_TKOREG     0x00180000
#define  SMP_TKOSPACE   0x00100000
#define  SMP_MEMSPACE   0x00000000

#define  SMP_SEQ_NOP         0x18

#define  SMP_SEQ_AC_ALL      0x00
#define  SMP_SEQ_AC_CMP      0x01
#define  SMP_SEQ_AC_SUB      0x02
#define  SMP_SEQ_AC_SCH_ALL  0x04
#define  SMP_SEQ_AC_SCH_CMP  0x05
#define  SMP_SEQ_AC_SCH_SUB  0x06
#define  SMP_SEQ_AC_REJ      0x1F

#define  SMP_SEQ_GO_ALL      0x10
#define  SMP_SEQ_GO_CMP      0x11
#define  SMP_SEQ_GO_SUB      0x12
#define  SMP_SEQ_GO_SCH_ALL  0x14
#define  SMP_SEQ_GO_SCH_CMP  0x15
#define  SMP_SEQ_GO_SCH_SUB  0x16
#define  SMP_SEQ_GO_REJ      0x1F

#define  SMP_BCR_SOFTSWITCH  0x01e0
#define  SMP_BCR_SWENABLE    0x0020
#define  SMP_BCR_SWAUTOCLR   0x0010
#define  SMP_BCR_SWCTMASK    0x0002
#define  SMP_BCR_SWSZMASK    0x0001
#define  SMP_BCR_SWMASK      (SMP_BCR_SWCTMASK|SMP_BCR_SWSZMASK)
#define  SMP_BCR_SWFLAG      0x80

#define  SMP_STAR_BUSY       0x20

#define  SMP_DSR_OVERFLOW    0x80000000

//
#define  SMP_GO        ((0x01)<<5)
#define  SMP_RSET      ((0x01)<<6)
#define  SMP_PSEL(x)   ((x)&0x1f)
#define  SMP_F(x)      (((x)&0x0f)<<7)
#define  SMP_BC(x)     (((x)&0x01)<<18)
#define  SMP_MA(m)     (((m)&0x1f)<<13)
#define  SMP_SA(m)     (((s)&0x7ff)<<2)
#define  SMP_MASA(m,s) (SMP_MA(m)|SMP_SA(s))

#define  SMP_TKOF(x)    (((x)&0x0f)<<7)
#define  SMP_TKOBC(x)   (((x)&0x01)<<18)
#define  SMP_TKOMA(x)   (((x)&0x1f)<<13)
#define  SMP_TKOSA(x)   (((x)&0x7ff)<<2)

#define  SMP_XQ_DATA(x) (((x)>>16)&0x01)
#define  SMP_XQ_STAR(x) (((x)>>6)&0x01)

#define  SMP_REG_INDEX(x)  ((x)>>2)
#define  SMP_ADDR_INDEX(x) ((x)>>2)


//// class Smp
class Smp {
public:
  // constructor/destructor
  Smp();
  Smp( unsigned int baseaddr, int debugLevel = 0 );
  Smp( const Smp& );
  ~Smp();
  //
  bool    init( unsigned int baseaddr, int debugLevel = 0 );
  bool    term();
  //
  Smp&  operator=( const Smp& );
  //
  const unsigned int*  map() { return m_map; }
  const unsigned int*  dmaPtr() { return (const unsigned int*)m_fdmaptr; }
  //
  void          tkoCommand( int reg,int cmd );
  unsigned int  tkoStatus( int reg );
  unsigned int  tkoRead(  int MA,int SA );
  int           tkoReadB( int MA, int SA, int size, unsigned int* data );
  void          tkoWrite( int MA, int SA, int BC, unsigned int data );
  unsigned int  smpRead( unsigned int index );
  int           smpReadB( unsigned int index, int size, unsigned int* data );
  int           smpReadD( unsigned int index, int size );
  //
  void          bufferSwitch( int mask );
  bool          bufferSwitchWait( int timeout_count );
  bool          busy();
  bool          busyWait( int timeout_count );
  
private:
  int            m_debugLevel;
  int            m_baseaddr;
  int            m_fd32;
  unsigned int*  m_map;
  int            m_fd32fdma;
  ioctl_map_t    m_fdma;
  char*          m_fdmaptr;
  //
};


//
#endif  //SMP_H_INCLUDED

